% Autor:
% Datum: 10.03.03


% Variablenwert
varWert(_, [], 'undefiniert').
varWert(V, [wert(V, W) | _], W).
varWert(V, [wert(X, _) | R], W) :- V \= X, varWert(V, R, W).

% Termwert
termWert(V, Z, V) :- integer(V).
termWert(V, Z, W) :- atom(V), varWert(V, Z, W).
termWert(T1 + T2, Z, W) :-
  termWert(T1, Z, W1), termWert(T2, Z, W2), W is W1+W2.
termWert(T1 - T2, Z, W) :-
  termWert(T1, Z, W1), termWert(T2, Z, W2), W is W1-W2.
termWert(T1 * T2, Z, W) :-
  termWert(T1, Z, W1), termWert(T2, Z, W2), W is W1*W2.
termWert(T1 / T2, Z, W) :-
  termWert(T1, Z, W1), termWert(T2, Z, W2), W is W1 // W2.
termWert(T1 : T2, Z, W) :-
  termWert(T1, Z, W1), termWert(T2, Z, W2), W is W1 mod W2.

% Wert einer Bedingung
booleWert(T1 = T2, Z) :-
  termWert(T1, Z, W1), termWert(T2, Z, W2), W1 =:= W2.
booleWert(T1 =\= T2, Z) :-
  termWert(T1, Z, W1), termWert(T2, Z, W2), W1 =\= W2.
booleWert(T1 < T2, Z) :-
  termWert(T1, Z, W1), termWert(T2, Z, W2), W1 < W2.
booleWert(T1 > T2, Z) :-
  termWert(T1, Z, W1), termWert(T2, Z, W2), W1 > W2.
booleWert(T1 =< T2, Z) :-
  termWert(T1, Z, W1), termWert(T2, Z, W2), W1 =< W2.
booleWert(T1 >= T2, Z) :-
  termWert(T1, Z, W1), termWert(T2, Z, W2), W1 >= W2.



% Neuer Zustand
neuerZustand(V, W, [], [wert(V, W)]).
neuerZustand(V, W, [wert(V, _) | R], [wert(V, W) | R]).
neuerZustand(V, W, [wert(X, WAlt) | R], [wert(X, WAlt) | RNeu]) :-
   V \= X, neuerZustand(V, W, R, RNeu).

% Zuweisung ausfhren
ausfuehren(let(X, T), ZAlt, ZNeu) :-
  termWert(T, ZAlt, W), neuerZustand(X, W, ZAlt, ZNeu).
% Sequenz ausfhren
ausfuehren([], ZAlt, ZAlt).
ausfuehren([Anw | R], ZAlt, ZNeu) :-
  ausfuehren(Anw, ZAlt, Z), ausfuehren(R, Z, ZNeu).
% Fallunterscheidung ausfhren
ausfuehren(if(Bed, Anw1, Anw2), ZAlt, ZNeu) :-
  booleWert(Bed, ZAlt), ausfuehren(Anw1, ZAlt, ZNeu).
ausfuehren(if(Bed, Anw1, Anw2), ZAlt, ZNeu) :-
  not(booleWert(Bed, ZAlt)), ausfuehren(Anw2, ZAlt, ZNeu).
% Wiederholung ausfhren
ausfuehren(while(Bed, Anw), ZAlt, ZNeu) :-
  booleWert(Bed, ZAlt),
  ausfuehren(Anw, ZAlt, Z),
  ausfuehren(while(Bed, Anw), Z, ZNeu).
ausfuehren(while(Bed, Anw), ZAlt, ZAlt) :-
  not(booleWert(Bed, ZAlt)).


test_varWert(X, W) :-
  varWert(X, [wert(x,2), wert(y,5), wert(z, 7)], W).

test1_termWert(T, W) :-
  termWert(T, [wert(x,2), wert(y,5), wert(z, 7)], W).

test1_booleWert(T) :-
  booleWert(T, [wert(x,2), wert(y,5), wert(z, 7)]).

  
test_neuerZustand(X, W, Z) :-
  neuerZustand(X, W, [wert(x,2), wert(y,5), wert(z, 7)], Z).

test1_ausfuehren(Z) :-
  ausfuehren(let(z, y*z), [wert(x,2), wert(y,5), wert(z, 7)], Z).

test2_ausfuehren(Z) :-
  ausfuehren([let(h, y), let(y, z), let(z, h)],
             [wert(x,2), wert(y,5), wert(z, 7)],
             Z).
test3_ausfuehren(Z) :-
  ausfuehren(if(x<y, let(x,y), let(y,x)),
             [wert(x,2), wert(y,5), wert(z, 7)],
             Z).

test4_ausfuehren :-
  ausfuehren(if(x>y, let(x,y), let(y,x)),
             [wert(x,2), wert(y,5), wert(z, 7)],
             Z),
             write(Z).

test5_ausfuehren(Z) :-
             ausfuehren(
                [let(y, 5), while(y>0, let(y, y-1))],[],Z).

test6_ausfuehren :-
             ausfuehren([let(y, 5), while(y>0, let(y, y-1))],[],Z),
             write(Z).

