% Autor: Klaus Becker
% Datum: 10.03.03

% Variablenwert
varWert(_, [], 'undefiniert').
varWert(V, [wert(V, W) | _R], W).
varWert(V, [wert(X, _W) | R], W) :- V \= X, varWert(V, R, W).

% Termwert
termWert(V, _Z, V) :- integer(V).
termWert(V, Z, W) :- atom(V), varWert(V, Z, W).
termWert(T1 + T2, Z, W) :-
  termWert(T1, Z, W1), termWert(T2, Z, W2), W is W1+W2.
termWert(T1 - T2, Z, W) :-
  termWert(T1, Z, W1), termWert(T2, Z, W2), W is W1-W2.
termWert(T1 * T2, Z, W) :-
  termWert(T1, Z, W1), termWert(T2, Z, W2), W is W1*W2.
termWert(T1 / T2, Z, W) :-
  termWert(T1, Z, W1), termWert(T2, Z, W2), W is W1 // W2.
termWert(T1 : T2, Z, W) :-
  termWert(T1, Z, W1), termWert(T2, Z, W2), W is W1 mod W2.
termWert(-T, Z, W) :-
  termWert(T, Z, W1), W is -W1.

% Wert einer Bedingung
booleWert(T1 = T2, Z) :-
  termWert(T1, Z, W1), termWert(T2, Z, W2), W1 =:= W2.
booleWert(T1 =\= T2, Z) :-
  termWert(T1, Z, W1), termWert(T2, Z, W2), W1 =\= W2.
booleWert(T1 < T2, Z) :-
  termWert(T1, Z, W1), termWert(T2, Z, W2), W1 < W2.
booleWert(T1 > T2, Z) :-
  termWert(T1, Z, W1), termWert(T2, Z, W2), W1 > W2.
booleWert(T1 =< T2, Z) :-
  termWert(T1, Z, W1), termWert(T2, Z, W2), W1 =< W2.
booleWert(T1 >= T2, Z) :-
  termWert(T1, Z, W1), termWert(T2, Z, W2), W1 >= W2.



% Neuer Zustand
neuerZustand(V, W, [], [wert(V, W)]).
neuerZustand(V, W, [wert(V, _) | R], [wert(V, W) | R]).
neuerZustand(V, W, [wert(X, WAlt) | R], [wert(X, WAlt) | RNeu]) :-
   V \= X, neuerZustand(V, W, R, RNeu).

% Zuweisung ausfhren
ausfuehren(let(X, T), ZAlt, ZNeu) :-
  termWert(T, ZAlt, W), neuerZustand(X, W, ZAlt, ZNeu).
% Sequenz ausfhren
ausfuehren([], ZAlt, ZAlt).
ausfuehren([Anw | R], ZAlt, ZNeu) :-
  ausfuehren(Anw, ZAlt, Z), ausfuehren(R, Z, ZNeu).
% Fallunterscheidung ausfhren
ausfuehren(if(Bed, Anw1, _Anw2), ZAlt, ZNeu) :-
  booleWert(Bed, ZAlt), ausfuehren(Anw1, ZAlt, ZNeu).
ausfuehren(if(Bed, _Anw1, Anw2), ZAlt, ZNeu) :-
  not(booleWert(Bed, ZAlt)), ausfuehren(Anw2, ZAlt, ZNeu).
% Wiederholung ausfhren
ausfuehren(while(Bed, Anw), ZAlt, ZNeu) :-
  booleWert(Bed, ZAlt),
  ausfuehren(Anw, ZAlt, Z),
  ausfuehren(while(Bed, Anw), Z, ZNeu).
ausfuehren(while(Bed, _Anw), ZAlt, ZAlt) :-
  not(booleWert(Bed, ZAlt)).

% Testprogramme

testprogramm1 :-
  ausfuehren([let(z, x), let(x, y), let(y, z)],
             [wert(x,2), wert(y,5)],
             Z),
  write(Z).

testprogramm2 :-
  ausfuehren([let(x, x-y), let(y, x+y), let(x, y-x)],
             [wert(x,2), wert(y,5)],
             Z),
  write(Z).

testprogramm3 :-
  ausfuehren(if(x > y, let(betrag, x-y), let(betrag, y-x)),
             [wert(x,2), wert(y,5)],
             Z),
  write(Z).
  
testprogramm4 :-
  ausfuehren(while(n > 0, [let(p, p*b), let(n, n-1)]),
             [wert(b,2), wert(n,3), wert(p,1)],
             Z),
  write(Z).

testprogramm5 :-
  ausfuehren([let(p,1), while(u > 0, [if(u:2 = 1, [let(u, u-1), let(p, p*b)], []), let(u, u/2), let(b, b*b)])],
             [wert(b,2), wert(u,5)],
             Z),
  write(Z).
